@@ -4,7 +4,14 @@ duplicationMatrix <- function(n, diag = TRUE){
   mat[lower.tri(mat,diag=diag)] <- seq_len(sum(lower.tri(mat,diag=diag)))
   mat[upper.tri(mat,diag=diag)] <- t(mat)[upper.tri(mat,diag=diag)]
   inds <- c(mat)
-  Matrix::sparseMatrix(i=seq_len(n^2)[inds!=0],j=inds[inds!=0],dims = c(n^2,sum(lower.tri(mat,diag=diag))))
+  res <- Matrix::sparseMatrix(i=seq_len(n^2)[inds!=0],j=inds[inds!=0],dims = c(n^2,sum(lower.tri(mat,diag=diag))))
+  # if (diag){
+  #   res <- as(res, "indMatrix")
+  # } else {
+  #   res <- as(res, "dgCMatrix")
+  # }
+  res <- as(res, "dgCMatrix")
+  return(res)
 }
 
 # Fast elimination matrix:
@@ -12,7 +19,10 @@ eliminationMatrix <- function(n, diag = TRUE){
   mat <- matrix(0,n,n)
   mat[lower.tri(mat,diag=diag)] <- seq_len(sum(lower.tri(mat,diag=diag)))
   inds <- c(mat)
-  Matrix::sparseMatrix(j=seq_len(n^2)[inds!=0],i=inds[inds!=0],dims = c(sum(lower.tri(mat,diag=diag)),n^2))
+  res <- Matrix::sparseMatrix(j=seq_len(n^2)[inds!=0],i=inds[inds!=0],dims = c(sum(lower.tri(mat,diag=diag)),n^2))
+  # res <- as(res, "indMatrix")
+  res <- as(res, "dgCMatrix")
+  return(res)
 }
 
 # Diagonalization matrix:
@@ -25,7 +35,8 @@ diagonalizationMatrix <- function(n){
   # browser()
   # seq(1, n^2, length = n)
     # foo <- as(Reduce(cbind,lapply(1:n,function(x)c(A(x,n)))),"sparseMatrix")
-  sparseMatrix(i = seq(1, n^2, length = n),j=seq_len(n),dims = c(n^2,n))
+  mat <- sparseMatrix(i = seq(1, n^2, length = n),j=seq_len(n),dims = c(n^2,n))
+  as(mat, "dgCMatrix")
 }
 
 # Basis matrix:
@@ -33,14 +44,16 @@ basisVector <- function(x,n){
   sparseMatrix(i = x,j=1,dims=c(n,1))
 }
 basisMatrix <- function(n){
-  do.call(rbind,lapply(seq_len(n),function(i,n){
+  mat <- do.call(rbind,lapply(seq_len(n),function(i,n){
     kronecker(Diagonal(n), basisVector(i,n))
   },n=n))
+  as(mat, "dgCMatrix")
 }
 
 # E matrix:
 Emat <- function(n,mat = diag(n)){
-  do.call(rbind,lapply(seq_len(n),function(i,n){
+  mat <- do.call(rbind,lapply(seq_len(n),function(i,n){
     kronecker(Diagonal(n),mat[,i])
   },n=n))
+  as(mat, "dgCMatrix")
 }
\ No newline at end of file
